//
//  PushNotificationRegistrationManager.swift
//  SampleApp
//
//  Created by Federico Zanetello on 5/27/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import AmitySDK

final class PushNotificationRegistrationManager {
    private unowned let client: AmityClient

    init(client: AmityClient) {
        self.client = client
    }

    func register(token: String, completion: @escaping (Result<Void, Error>) -> Void) {
        UserDefaults.standard.isRegisterdForPushNotification = true
        Task { @MainActor in
            do {
                let _ = try await client.registerPushNotification(withDeviceToken: token)
                completion(.success(()))
            }catch {
                completion(.failure(error))
            }
        }
    }

    func unregisterUser(completion: @escaping (Result<Void, Error>) -> Void) {
        guard let currentUserId = client.currentUserId else { return }
        Task { @MainActor in
            do {
                let _ = try await client.unregisterPushNotification(forUserId: currentUserId)
                completion(.success(()))
            }catch {
                completion(.failure(error))
            }
        }
    }

    func unregisterDevice(completion: @escaping (Result<Void, Error>) -> Void) {
        Task { @MainActor in
            do {
                let _ = try await client.unregisterPushNotification(forUserId: nil)
                completion(.success(()))
            }catch {
                completion(.failure(error))
            }
        }
    }
}
